@@ -1,5 +1,23 @@
 #!/bin/bash
 
+## %CopyrightBegin%
+##
+## Copyright Ericsson AB 2024. All Rights Reserved.
+##
+## Licensed under the Apache License, Version 2.0 (the "License");
+## you may not use this file except in compliance with the License.
+## You may obtain a copy of the License at
+##
+##     http://www.apache.org/licenses/LICENSE-2.0
+##
+## Unless required by applicable law or agreed to in writing, software
+## distributed under the License is distributed on an "AS IS" BASIS,
+## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+## See the License for the specific language governing permissions and
+## limitations under the License.
+##
+## %CopyrightEnd%
+
 set -eo pipefail
 
 BASE_BRANCH="$1"
@@ -22,15 +40,15 @@ fi
 
 case "${BASE_TAG}" in
     *i386-debian-base)
-        BASE="i386/debian:bullseye"
+        BASE="i386/debian:bookworm"
         BASE_TYPE=debian-base
         ;;
     *debian-base)
-        BASE="debian:bullseye"
+        BASE="debian:bookworm"
         BASE_TYPE=debian-base
         ;;
     *ubuntu-base)
-        BASE="ubuntu:20.04"
+        BASE="ubuntu:22.04"
         BASE_TYPE=ubuntu-base
         ;;
 esac
@@ -47,9 +65,10 @@ elif [ -f "otp_docker_base/otp_docker_base.tar" ]; then
     echo "BASE_BUILD=loaded" >> $GITHUB_OUTPUT
 else
     if [ "${BASE_USE_CACHE}" != "false" ]; then
-        docker pull "${BASE_TAG}:${BASE_BRANCH}"
-        docker tag "${BASE_TAG}:${BASE_BRANCH}" "${BASE_TAG}:latest"
-        BASE_CACHE="--cache-from ${BASE_TAG}"
+        if docker pull "${BASE_TAG}:${BASE_BRANCH}"; then
+            docker tag "${BASE_TAG}:${BASE_BRANCH}" "${BASE_TAG}:latest"
+            BASE_CACHE="--cache-from ${BASE_TAG}"
+        fi
     fi
 
     BASE_IMAGE_ID=$(docker images -q "${BASE_TAG}:latest")
