@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2013-2022. All Rights Reserved.
+%% Copyright Ericsson AB 2013-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -31,9 +31,18 @@
          run/1]).
 
 %% common_test wrapping
--export([suite/0,
+-export([
+         %% Framework functions
+         suite/0,
          all/0,
-         traffic/1]).
+         init_per_suite/1,
+         end_per_suite/1,
+         init_per_testcase/2,
+         end_per_testcase/2,
+         
+         %% The test cases
+         traffic/1
+        ]).
 
 %% internal
 -export([send_unknown_application/1,
@@ -60,9 +69,11 @@
 -include("diameter_gen_base_rfc6733.hrl").
 %% Use the fact that STR/STA is identical in RFC's 3588 and 6733.
 
+-include("diameter_util.hrl").
+
+
 %% ===========================================================================
 
--define(util, diameter_util).
 -define(testcase(), get(?MODULE)).
 
 -define(L, atom_to_list).
@@ -95,6 +106,10 @@
 
 -define(LOGOUT, ?'DIAMETER_BASE_TERMINATION-CAUSE_LOGOUT').
 
+-define(XL(F),    ?XL(F, [])).
+-define(XL(F, A), ?LOG("D3XS", F, A)).
+
+
 %% ===========================================================================
 
 suite() ->
@@ -103,9 +118,50 @@ suite() ->
 all() ->
     [traffic].
 
+
+init_per_suite(Config) ->
+    ?XL("init_per_suite -> entry with"
+        "~n   Config: ~p", [Config]),
+    ?DUTIL:init_per_suite(Config).
+
+end_per_suite(Config) ->
+    ?XL("end_per_suite -> entry with"
+        "~n   Config: ~p", [Config]),
+    ?DUTIL:end_per_suite(Config).
+
+
+%% This test case can take a *long* time, so if the machine is too slow, skip
+init_per_testcase(Case, Config) when is_list(Config) ->
+    ?XL("init_per_testcase(~w) -> entry with"
+        "~n   Config: ~p"
+        "~n   => check factor", [Case, Config]),
+    Key = dia_factor,
+    case lists:keysearch(Key, 1, Config) of
+        {value, {Key, Factor}} when (Factor > 10) ->
+            ?XL("init_per_testcase(~w) -> Too slow (~w) => SKIP",
+                [Case, Factor]),
+            {skip, {machine_too_slow, Factor}};
+        _ ->
+            ?XL("init_per_testcase(~w) -> run test", [Case]),
+            Config
+    end;
+init_per_testcase(Case, Config) ->
+    ?XL("init_per_testcase(~w) -> entry", [Case]),
+    Config.
+
+
+end_per_testcase(Case, Config) when is_list(Config) ->
+    ?XL("end_per_testcase(~w) -> entry", [Case]),
+    Config.
+
+
+%% ===========================================================================
+
 traffic(_Config) ->
+    ?XL("traffic -> entry"),
     run().
 
+
 %% ===========================================================================
 
 tc() ->
@@ -123,44 +179,68 @@ tc() ->
 %% run/0
 
 run() ->
-    ?util:run([[{{?MODULE, run, [{E,D}]}, 60000} || E <- ?ERRORS,
-                                                    D <- ?RFCS]]).
+    ?XL("run -> entry"),
+    ?RUN([[{{?MODULE, run, [{E,D}]}, 60000} || E <- ?ERRORS,
+                                               D <- ?RFCS]]).
 
 %% run/1
 
 run({F, [_,_] = G}) ->
+    ?XL("run -> entry with"
+        "~n   F: ~p"
+        "~n   G: ~p", [F, G]),    
     put(?MODULE, F),
     apply(?MODULE, F, [G]);
 
 run({E,D}) ->
+    ?XL("run -> entry with"
+        "~n   E: ~p"
+        "~n   D: ~p", [E, D]),
     try
         run([E,D])
     after
+        ?XL("run(after) -> stop diameter app"),
         ok = diameter:stop()
     end;
 
 run([Errors, RFC] = G) ->
+    ?XL("run -> entry with"
+        "~n   Errors: ~p"
+        "~n   RFC:    ~p", [Errors, RFC]),
     Name = ?L(Errors) ++ "," ++ ?L(RFC),
+    ?XL("run -> start diameter app"),
     ok = diameter:start(),
+    ?XL("run -> start service 'server' (~p)", [Name]),
     ok = diameter:start_service(?SERVER, ?SERVICE(Name, Errors, RFC)),
+    ?XL("run -> start service 'client'"),
     ok = diameter:start_service(?CLIENT, ?SERVICE(?CLIENT,
                                                   callback,
                                                   rfc6733)),
-    LRef = ?util:listen(?SERVER, tcp),
-    ?util:connect(?CLIENT, tcp, LRef),
-    ?util:run([{?MODULE, run, [{F,G}]} || F <- tc()]),
+    ?XL("run -> (server) listen"),
+    LRef = ?LISTEN(?SERVER, tcp),
+    ?XL("run -> (client) connect"),
+    ?CONNECT(?CLIENT, tcp, LRef),
+    ?XL("run -> run"),
+    ?RUN([{?MODULE, run, [{F,G}]} || F <- tc()]),
+    ?XL("run -> \"check\" counters"),
     _ = counters(G),
+    ?XL("run -> remove 'client' transport"),
     ok = diameter:remove_transport(?CLIENT, true),
+    ?XL("run -> remove 'server' transport"),
     ok = diameter:remove_transport(?SERVER, true),
+    ?XL("run -> stop service 'server'"),
     ok = diameter:stop_service(?SERVER),
-    ok = diameter:stop_service(?CLIENT).
+    ?XL("run -> stop service 'client'"),
+    ok = diameter:stop_service(?CLIENT),
+    ?XL("run -> done"),
+    ok.
 
 %% counters/1
 %%
 %% Check that counters are as expected.
 
 counters([_Errors, _RFC] = G) ->
-    [] = ?util:run([[fun counters/3, K, S, G]
+    [] = ?RUN([[fun counters/3, K, S, G]
                     || K <- [statistics, transport, connections],
                        S <- [?CLIENT, ?SERVER]]).
 
@@ -193,26 +273,56 @@ stats(?CLIENT, E, rfc3588, L)
      {{{0,275,0},recv,{'Result-Code',5005}},1}]
         = L;
 
-stats(?SERVER, E, rfc3588, L)
+stats(?SERVER = Svc, E, rfc3588 = RFC, L)
   when E == answer;
        E == answer_3xxx ->
-    [{{{unknown,0},send},2},
-     {{{unknown,1},recv},1},
-     {{{0,257,0},send},1},
-     {{{0,257,1},recv},1},
-     {{{0,275,0},send},6},
-     {{{0,275,1},recv},8},
-     {{{unknown,0},send,{'Result-Code',3001}},1},
-     {{{unknown,0},send,{'Result-Code',3007}},1},
-     {{{unknown,1},recv,error},1},
-     {{{0,257,0},send,{'Result-Code',2001}},1},
-     {{{0,275,0},send,{'Result-Code',2001}},1},
-     {{{0,275,0},send,{'Result-Code',3008}},2},
-     {{{0,275,0},send,{'Result-Code',3999}},1},
-     {{{0,275,0},send,{'Result-Code',5002}},1},
-     {{{0,275,0},send,{'Result-Code',5005}},1},
-     {{{0,275,1},recv,error},5}]
-        = L;
+    %% [{{{unknown,0},send},2},
+    %%  {{{unknown,1},recv},1},
+    %%  {{{0,257,0},send},1},
+    %%  {{{0,257,1},recv},1},
+    %%  {{{0,275,0},send},6},
+    %%  {{{0,275,1},recv},8},
+    %%  {{{unknown,0},send,{'Result-Code',3001}},1},
+    %%  {{{unknown,0},send,{'Result-Code',3007}},1},
+    %%  {{{unknown,1},recv,error},1},
+    %%  {{{0,257,0},send,{'Result-Code',2001}},1},
+    %%  {{{0,275,0},send,{'Result-Code',2001}},1},
+    %%  {{{0,275,0},send,{'Result-Code',3008}},2},
+    %%  {{{0,275,0},send,{'Result-Code',3999}},1},
+    %%  {{{0,275,0},send,{'Result-Code',5002}},1},
+    %%  {{{0,275,0},send,{'Result-Code',5005}},1},
+    %%  {{{0,275,1},recv,error},5}]
+    %% = L;
+    ?XL("~w(~p, ~w) -> (attempt to) verify answer: "
+        "~n   E: ~p"
+        "~n   L: ~p", [?FUNCTION_NAME, Svc, RFC, E, L]),
+    Expected = [{{{unknown,0},send},2},
+                {{{unknown,1},recv},1},
+                {{{0,257,0},send},1},
+                {{{0,257,1},recv},1},
+                {{{0,275,0},send},6},
+                {{{0,275,1},recv},8},
+                {{{unknown,0},send,{'Result-Code',3001}},1},
+                {{{unknown,0},send,{'Result-Code',3007}},1},
+                {{{unknown,1},recv,error},1},
+                {{{0,257,0},send,{'Result-Code',2001}},1},
+                {{{0,275,0},send,{'Result-Code',2001}},1},
+                {{{0,275,0},send,{'Result-Code',3008}},2},
+                {{{0,275,0},send,{'Result-Code',3999}},1},
+                {{{0,275,0},send,{'Result-Code',5002}},1},
+                {{{0,275,0},send,{'Result-Code',5005}},1},
+                {{{0,275,1},recv,error},5}],
+    case L of
+        Expected ->
+            ?XL("~w(~w) -> ok", [?FUNCTION_NAME, RFC]),
+            L;
+        _ ->
+            ?XL("~w(~w, ~w) -> wrong: "
+                "~n   L -- Expected: ~p"
+                "~n   Expected -- L: ~p",
+                [?FUNCTION_NAME, Svc, RFC, L -- Expected, Expected -- L]),
+            exit({wrong_answer, Svc, E, RFC})
+    end;
 
 stats(?CLIENT, answer, rfc6733, L) ->
     [{{{unknown,0},recv},2},
@@ -357,6 +467,7 @@ stats(?SERVER, callback, rfc6733, L) ->
 
 %% diameter answers.
 send_unknown_application([_,_]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #'diameter_base_answer-message'{'Result-Code' = 3007,
                                                  %% UNSUPPORTED_APPLICATION
                                     'Failed-AVP' = [],
@@ -369,10 +480,12 @@ send_unknown_application([_,_]) ->
 
 %% handle_request discards the request.
 send_unknown_command([callback, _]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     {error, timeout} = call();
 
 %% diameter answers.
 send_unknown_command([_,_]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #'diameter_base_answer-message'{'Result-Code' = 3001,
                                                  %% UNSUPPORTED_COMMAND
                                     'Failed-AVP' = [],
@@ -385,6 +498,7 @@ send_unknown_command([_,_]) ->
 
 %% Callback answers.
 send_ok([_,_]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #diameter_base_STA{'Result-Code' = 5002,  %% UNKNOWN_SESSION_ID
                        'Failed-AVP' = [],
                        'AVP' = []}
@@ -396,6 +510,7 @@ send_ok([_,_]) ->
 
 %% Callback answers.
 send_invalid_hdr_bits([callback, _]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #diameter_base_STA{'Result-Code' = 2001,  %% SUCCESS
                        'Failed-AVP' = [],
                        'AVP' = []}
@@ -403,6 +518,7 @@ send_invalid_hdr_bits([callback, _]) ->
 
 %% diameter answers.
 send_invalid_hdr_bits([_,_]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #'diameter_base_answer-message'{'Result-Code' = 3008, %% INVALID_HDR_BITS
                                     'Failed-AVP' = [],
                                     'AVP' = []}
@@ -414,6 +530,7 @@ send_invalid_hdr_bits([_,_]) ->
 
 %% diameter answers.
 send_missing_avp([answer, rfc6733]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #'diameter_base_answer-message'{'Result-Code' = 5005,  %% MISSING_AVP
                                     'Failed-AVP' = [_],
                                     'AVP' = []}
@@ -421,6 +538,7 @@ send_missing_avp([answer, rfc6733]) ->
 
 %% Callback answers.
 send_missing_avp([_,_]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #diameter_base_STA{'Result-Code' = 5005,  %% MISSING_AVP
                        'Failed-AVP' = [_],
                        'AVP' = []}
@@ -432,6 +550,7 @@ send_missing_avp([_,_]) ->
 
 %% diameter answers.
 send_ignore_missing_avp([answer, rfc6733]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #'diameter_base_answer-message'{'Result-Code' = 5005,  %% MISSING_AVP
                                     'Failed-AVP' = [_],
                                     'AVP' = []}
@@ -439,6 +558,7 @@ send_ignore_missing_avp([answer, rfc6733]) ->
 
 %% Callback answers, ignores the error
 send_ignore_missing_avp([_,_]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #diameter_base_STA{'Result-Code' = 2001,  %% SUCCESS
                        'Failed-AVP' = [],
                        'AVP' = []}
@@ -451,6 +571,7 @@ send_ignore_missing_avp([_,_]) ->
 
 %% RFC 6733 allows 5xxx in an answer-message.
 send_5xxx_missing_avp([_, rfc6733]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #'diameter_base_answer-message'{'Result-Code' = 5005,  %% MISSING_AVP
                                     'Failed-AVP' = [_],
                                     'AVP' = []}
@@ -458,10 +579,12 @@ send_5xxx_missing_avp([_, rfc6733]) ->
 
 %% RFC 3588 doesn't: sending answer fails.
 send_5xxx_missing_avp([_, rfc3588]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     {error, timeout} = call();
 
 %% Callback answers, ignores the error
 send_5xxx_missing_avp([_,_]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #diameter_base_STA{'Result-Code' = 2001,  %% SUCCESS
                        'Failed-AVP' = [],
                        'AVP' = []}
@@ -473,6 +596,7 @@ send_5xxx_missing_avp([_,_]) ->
 
 %% Callback answers with STA.
 send_double_error([callback, _]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #diameter_base_STA{'Result-Code' = 5005,  %% MISSING_AVP
                        'Failed-AVP' = [_],
                        'AVP' = []}
@@ -480,6 +604,7 @@ send_double_error([callback, _]) ->
 
 %% diameter answers with answer-message.
 send_double_error([_,_]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #'diameter_base_answer-message'{'Result-Code' = 3008, %% INVALID_HDR_BITS
                                     'Failed-AVP' = [],
                                     'AVP' = []}
@@ -491,6 +616,7 @@ send_double_error([_,_]) ->
 
 %% Callback answers.
 send_3xxx([_,_]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #'diameter_base_answer-message'{'Result-Code' = 3999,
                                     'Failed-AVP' = [],
                                     'AVP' = []}
@@ -503,10 +629,12 @@ send_3xxx([_,_]) ->
 %% Callback answers but fails since 5xxx isn't allowed in an RFC 3588
 %% answer-message.
 send_5xxx([_, rfc3588]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     {error, timeout} = call();
 
 %% Callback answers.
 send_5xxx([_,_]) ->
+    ?XL("~w -> entry", [?FUNCTION_NAME]),
     #'diameter_base_answer-message'{'Result-Code' = 5999,
                                     'Failed-AVP' = [],
                                     'AVP' = []}
@@ -516,13 +644,38 @@ send_5xxx([_,_]) ->
 
 call() ->
     Name = ?testcase(),
-    diameter:call(?CLIENT,
-                  ?DICT,
-                  #diameter_base_STR
-                  {'Termination-Cause' = ?LOGOUT,
-                   'Auth-Application-Id' = ?COMMON,
-                   'Class' = [?L(Name)]},
-                  [{extra, [Name]}]).
+    ?XL("call -> make diameter call with Name: ~p", [Name]),
+    %% There is a "bug" in diameter, which can cause this function to return
+    %% {error, timeout} even though only a fraction on the time has expired.
+    %% This is because the timer has in fact *not* expired. Instead what
+    %% has happened is the transport process has died and the selection
+    %% of a new transport fails (I think its a race causing the pick_peer
+    %% to return the same tranport process), at that error is converted to
+    %% a timeout error.
+    %% So, if this call returns {error, timeout} but only a fraction of the
+    %% time has passed we skip instead!
+    Timeout = 5000,
+    T1 = ?TS(),
+    case diameter:call(?CLIENT,
+                       ?DICT,
+                       #diameter_base_STR
+                       {'Termination-Cause' = ?LOGOUT,
+                        'Auth-Application-Id' = ?COMMON,
+                        'Class' = [?L(Name)]},
+                       [{extra, [Name]}, {timeout, Timeout}]) of
+        {error, timeout} = ERROR ->
+            T2 = ?TS(),
+            TDiff = T2 - T1,
+            if
+                TDiff < 100 ->
+                    exit({skip, {invalid_timeout, TDiff, Timeout}});
+                true ->
+                        ERROR
+            end;
+        R ->
+            R
+    end.
+
 
 %% ===========================================================================
 %% diameter callbacks
