@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2010-2016. All Rights Reserved.
+%% Copyright Ericsson AB 2010-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -34,26 +34,22 @@
 
 -module(diameter_relay_SUITE).
 
--export([suite/0,
-         all/0,
-         groups/0]).
+%% testcases, no common_test dependency
+-export([run/0]).
 
-%% testcases
--export([start/1,
-         start_services/1,
-         connect/1,
-         send1/1,
-         send2/1,
-         send3/1,
-         send4/1,
-         send_loop/1,
-         send_timeout_1/1,
-         send_timeout_2/1,
-         info/1,
-         counters/1,
-         disconnect/1,
-         stop_services/1,
-         stop/1]).
+%% common_test wrapping
+-export([
+         %% Framework functions
+         suite/0,
+         all/0,
+         init_per_suite/1,
+         end_per_suite/1,
+         init_per_testcase/2,
+         end_per_testcase/2,
+        
+         %% The test cases
+         parallel/1
+        ]).
 
 %% diameter callbacks
 -export([pick_peer/4,
@@ -65,9 +61,10 @@
 -include("diameter.hrl").
 -include("diameter_gen_base_rfc3588.hrl").
 
-%% ===========================================================================
+-include("diameter_util.hrl").
 
--define(util, diameter_util).
+
+%% ===========================================================================
 
 -define(ADDR, {127,0,0,1}).
 
@@ -112,141 +109,205 @@
 -define(LOGOUT, ?'DIAMETER_BASE_TERMINATION-CAUSE_LOGOUT').
 -define(AUTHORIZE_ONLY, ?'DIAMETER_BASE_RE-AUTH-REQUEST-TYPE_AUTHORIZE_ONLY').
 
+-define(RL(F),    ?RL(F, [])).
+-define(RL(F, A), ?LOG("DRELAYS", F, A)).
+
+
 %% ===========================================================================
 
 suite() ->
-    [{timetrap, {seconds, 60}}].
+    [{timetrap, {seconds, 30}}].
 
 all() ->
-    [start,
-     start_services,
-     connect,
-     {group, all},
-     counters,
-     {group, all, [parallel]},
-     disconnect,
-     stop_services,
-     stop].
+    [parallel].
 
-groups() ->
-    [{all, [], tc()}].
 
-%% Traffic cases run when services are started and connections
-%% established.
-tc() ->
-    [send1,
-     send2,
-     send3,
-     send4,
-     send_loop,
-     send_timeout_1,
-     send_timeout_2,
-     info].
+init_per_suite(Config) ->
+    ?RL("init_per_suite -> entry with"
+        "~n   Config: ~p", [Config]),
+    ?DUTIL:init_per_suite(Config).
+
+end_per_suite(Config) ->
+    ?RL("end_per_suite -> entry with"
+        "~n   Config: ~p", [Config]),
+    ?DUTIL:end_per_suite(Config).
+
+
+%% This test case can take a *long* time, so if the machine is too slow, skip
+init_per_testcase(parallel = Case, Config) when is_list(Config) ->
+    ?RL("init_per_testcase(~w) -> check factor", [Case]),
+    Key = dia_factor,
+    case lists:keysearch(Key, 1, Config) of
+        {value, {Key, Factor}} when (Factor > 10) ->
+            ?RL("init_per_testcase(~w) -> Too slow (~w) => SKIP",
+                [Case, Factor]),
+            {skip, {machine_too_slow, Factor}};
+        _ ->
+            ?RL("init_per_testcase(~w) -> run test", [Case]),
+            Config
+    end;
+init_per_testcase(Case, Config) ->
+    ?RL("init_per_testcase(~w) -> entry", [Case]),
+    Config.
+
+
+end_per_testcase(Case, Config) when is_list(Config) ->
+    ?RL("end_per_testcase(~w) -> entry", [Case]),
+    Config.
+
 
 %% ===========================================================================
-%% start/stop testcases
 
-start(_Config) ->
-    ok = diameter:start().
+parallel(_Config) ->
+    ?RL("parallel -> entry"),
+    Res = run(),
+    ?RL("parallel -> done when"
+        "~n   Res: ~p", [Res]),
+    Res.
 
-start_services(_Config) ->
+
+%% ===========================================================================
+
+%% run/0
+
+run() ->
+    ok = diameter:start(),
+    try
+        ?RUN([{fun traffic/0, 20000}])
+    after
+        ok = diameter:stop()
+    end.
+
+%% traffic/0
+
+traffic() ->
+    ?RL("traffic -> start services"),
+    Servers = start_services(),
+    ?RL("traffic -> connect"),
+    Conns = connect(Servers),
+    ?RL("traffic -> send"),
+    [] = send(),
+    ?RL("traffic -> check counters"),
+    [] = counters(),
+    ?RL("traffic -> disconnect"),
+    [] = disconnect(Conns),
+    ?RL("traffic -> stop services"),
+    [] = stop_services(),
+    ?RL("traffic -> done"),
+    ok.
+
+start_services() ->
     [S1,S2,S3,S4] = [server(N, ?DICT_COMMON) || N <- [?SERVER1,
                                                       ?SERVER2,
                                                       ?SERVER3,
                                                       ?SERVER4]],
     [R1,R2] = [server(N, ?DICT_RELAY) || N <- [?RELAY1, ?RELAY2]],
 
+    ?RL("server -> try start service ~s", [?CLIENT]),
     ok = diameter:start_service(?CLIENT, ?SERVICE(?CLIENT, ?DICT_COMMON)),
 
-    {save_config, [{?RELAY1, [S1,S2,R2]},
-                   {?RELAY2, [S3,S4]},
-                   {?CLIENT, [R1,R2]}]}.
+    [{?RELAY1, [S1,S2,R2]}, {?RELAY2, [S3,S4]}, {?CLIENT, [R1,R2]}].
 
-connect(Config) ->
-    {_, Conns} = proplists:get_value(saved_config, Config),
+connect(Servers) ->
+    lists:flatmap(fun({CN,Ss}) -> connect(CN, Ss) end, Servers).
 
-    ?util:write_priv(Config,
-                     "cfg",
-                     lists:flatmap(fun({CN,Ss}) -> connect(CN, Ss) end,
-                                   Conns)).
+disconnect(Conns) ->
+    [{T,C} || C <- Conns,
+              T <- [break(C)],
+              T /= ok].
 
-disconnect(Config) ->
-    [] = [{T,C} || C <- ?util:read_priv(Config, "cfg"),
-                   T <- [break(C)],
-                   T /= ok].
+stop_services() ->
+    [{H,T} || H <- ?SERVICES,
+              T <- [diameter:stop_service(H)],
+              T /= ok].
 
-stop_services(_Config) ->
-    [] = [{H,T} || H <- ?SERVICES,
-                   T <- [diameter:stop_service(H)],
-                   T /= ok].
+%% Traffic cases run when services are started and connections
+%% established.
+send() ->
+    ?RUN([[fun(X) ->
+                   ?RL("send:fun -> entry with ~w", [X]),
+                   traffic(X)
+           end, T] || T <- [send1,
+                            send2,
+                            send3,
+                            send4,
+                            send_loop,
+                            send_timeout_1,
+                            send_timeout_2,
+                            info]]).
 
-stop(_Config) ->
-    ok = diameter:stop().
 
 %% ----------------------------------------
 
 break({{CN,CR},{SN,SR}}) ->
     try
-        ?util:disconnect(CN,CR,SN,SR)
+        ?DISCONNECT(CN,CR,SN,SR)
     after
         diameter:remove_transport(SN, SR)
     end.
 
 server(Name, Dict) ->
+    ?RL("server -> try start service ~s", [Name]),
     ok = diameter:start_service(Name, ?SERVICE(Name, Dict)),
-    {Name, ?util:listen(Name, tcp)}.
+    {Name, ?LISTEN(Name, tcp)}.
 
 connect(Name, Refs) ->
-    [{{Name, ?util:connect(Name, tcp, LRef)}, T} || {_, LRef} = T <- Refs].
+    [{{Name, ?CONNECT(Name, tcp, LRef)}, T} || {_, LRef} = T <- Refs].
 
 %% ===========================================================================
 %% traffic testcases
 
 %% Send an STR intended for a specific server and expect success.
-send1(_Config) ->
-    call(?SERVER1).
-send2(_Config) ->
-    call(?SERVER2).
-send3(_Config) ->
-    call(?SERVER3).
-send4(_Config) ->
-    call(?SERVER4).
+traffic(send1) ->
+    call(?SERVER1);
+traffic(send2) ->
+    call(?SERVER2);
+traffic(send3) ->
+    call(?SERVER3);
+traffic(send4) ->
+    call(?SERVER4);
 
 %% Send an ASR that loops between the relays (RELAY1 -> RELAY2 ->
 %% RELAY1) and expect the loop to be detected.
-send_loop(_Config) ->
+traffic(send_loop) ->
     Req = ['ASR', {'Destination-Realm', realm(?SERVER1)},
                   {'Destination-Host', ?SERVER1},
                   {'Auth-Application-Id', ?APP_ID}],
     #'diameter_base_answer-message'{'Result-Code' = ?LOOP_DETECTED}
-        = call(Req, [{filter, realm}]).
+        = call(Req, [{filter, realm}]);
 
 %% Send a RAR that is incorrectly routed and then discarded and expect
 %% different results depending on whether or not we or the relay
 %% timeout first.
-send_timeout_1(_Config) ->
+traffic(send_timeout_1) ->
     #'diameter_base_answer-message'{'Result-Code' = ?UNABLE_TO_DELIVER}
-        = send_timeout(7000).
-send_timeout_2(_Config) ->
-    {error, timeout} = send_timeout(3000).
+        = traffic({timeout, 7000});
+traffic(send_timeout_2) ->
+    {error, timeout} = traffic({timeout, 3000});
 
-send_timeout(Tmo) ->
+traffic({timeout, Tmo}) ->
     Req = ['RAR', {'Destination-Realm', realm(?SERVER1)},
                   {'Destination-Host', ?SERVER1},
                   {'Auth-Application-Id', ?APP_ID},
                   {'Re-Auth-Request-Type', ?AUTHORIZE_ONLY}],
-    call(Req, [{filter, realm}, {timeout, Tmo}]).
+    call(Req, [{filter, realm}, {timeout, Tmo}]);
 
-info(_Config) ->
+traffic(info) ->
     %% Wait for RELAY1 to have answered all requests, so that the
     %% suite doesn't end before all answers are sent and counted.
     receive after 6000 -> ok end,
-    [] = ?util:info().
+    [] = ?INFO().
 
-counters(_Config) ->
-    [] = ?util:run([[fun counters/2, K, S]
-                    || K <- [statistics, transport, connections],
-                       S <- ?SERVICES]).
+counters() ->
+    ?RUN([[fun(XK, XS) ->
+                   ?RL("counters:fun -> entry with"
+                       "~n   (X)Key: ~w"
+                       "~n   (X)Svc: ~p", [XK, XS]),                   
+                   counters(XK, XS)
+           end,
+           K, S]
+          || K <- [statistics, transport, connections],
+             S <- ?SERVICES]).
 
 counters(Key, Svc) ->
     counters(Key, Svc, [_|_] = diameter:service_info(Svc, Key)).
@@ -302,7 +363,7 @@ stats(?RELAY1, L) ->
                              %% RAR x 2 (send_timeout_[12])
      {{{0,257,0},recv},3},   %% CEA
      {{{0,257,0},send},1},   %%  "
-     {{{0,257,1},recv},1},   %% CER 
+     {{{0,257,1},recv},1},   %% CER
      {{{0,257,1},send},3},   %%  "
      {{{relay,0},recv,{'Result-Code',2001}},2},  %% STA x 2 (send[34])
      {{{relay,0},recv,{'Result-Code',3005}},1},  %% ASA     (send_loop)
