@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2010-2017. All Rights Reserved.
+%% Copyright Ericsson AB 2010-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -25,29 +25,30 @@
 
 -module(diameter_transport_SUITE).
 
--export([suite/0,
-         all/0,
-         groups/0,
-         init_per_suite/1,
-         end_per_suite/1]).
+%% all tests, no common_test dependency
+-export([run/0]).
 
-%% testcases
--export([start/1,
-         tcp_accept/1,
-         tcp_connect/1,
-         sctp_accept/1,
-         sctp_connect/1,
-         reconnect/1, reconnect/0,
-         stop/1]).
+%% common_test wrapping
+-export([
+         %% Framework functions
+         suite/0,
+         all/0,
+         init_per_suite/1,
+         end_per_suite/1,
+        
+         %% The test cases
+         parallel/1
+        ]).
 
 -export([accept/1,
          connect/1,
+         reconnect/1,
          init/2]).
 
 -include_lib("kernel/include/inet_sctp.hrl").
 -include("diameter.hrl").
 
--define(util, diameter_util).
+-include("diameter_util.hrl").
 
 %% Corresponding to diameter_* transport modules.
 -define(TRANSPORTS, [tcp, sctp]).
@@ -83,94 +84,114 @@
 %% Messages from gen_sctp.
 -define(SCTP(Sock, Data), {sctp, Sock, _, _, Data}).
 
+
+-define(TL(F),    ?TL(F, [])).
+-define(TL(F, A), ?LOG("DTRANSPS", F, A)).
+
+
 %% ===========================================================================
+%% common_test wrapping
 
 suite() ->
-    [{timetrap, {seconds, 15}}].
+    [{timetrap, {seconds, 270}}].
 
 all() ->
-    [start,
-     {group, all},
-     {group, all, [parallel]},
-     stop].
+    [parallel].
 
-groups() ->
-    [{all, [], tc()}].
-
-tc() ->
-    [tcp_accept,
-     tcp_connect,
-     sctp_accept,
-     sctp_connect,
-     reconnect].
 
 init_per_suite(Config) ->
-    [{sctp, ?util:have_sctp()} | Config].
+    ?DUTIL:init_per_suite(Config).
+
+end_per_suite(Config) ->
+    ?DUTIL:end_per_suite(Config).
+
+
+parallel(_) ->
+    ?TL("parallel -> entry"),
+    Res = run(),
+    ?TL("parallel -> done when"
+        "~n   Res: ~p", [Res]),
+    Res.
 
-end_per_suite(_Config) ->
-    ok.
 
 %% ===========================================================================
 
-start(_Config) ->
-    ok = diameter:start().
+%% run/0
+
+run() ->
+    ok = diameter:start(),
+    try
+        ?RUN([[fun run/1, {P,F}]
+              || P <- [sctp || ?HAVE_SCTP()] ++ [tcp],
+                 F <- [connect, accept, reconnect]])
+    after
+        diameter:stop()
+    end.
+
+%% run/1
+
+run({Prot, reconnect}) ->
+    ?TL("run(reconnect) -> entry with"
+        "~n   Prot: ~p", [Prot]),
+    Res = reconnect(Prot),
+    ?TL("run(reconnect) -> done when"
+        "~n   Res: ~p", [Res]),
+    Res;
+
+run({Prot, accept}) ->
+    ?TL("run(accept) -> entry with"
+        "~n   Prot: ~p", [Prot]),
+    Res = accept(Prot),
+    ?TL("run(accept) -> done when"
+        "~n   Res: ~p", [Res]),
+    Res;
+
+run({Prot, connect}) ->
+    ?TL("run(connect) -> entry with"
+        "~n   Prot: ~p", [Prot]),
+    Res = connect(Prot),
+    ?TL("run(connect) -> done when"
+        "~n   Res: ~p", [Res]),
+    Res.
 
-stop(_Config) ->
-    ok = diameter:stop().
 
 %% ===========================================================================
-%% tcp_accept/1
-%% sctp_accept/1
+%% accept/1
 %%
 %% diameter transport accepting, test code connecting.
 
-tcp_accept(_) ->
-    accept(tcp).
-
-sctp_accept(Config) ->
-    case lists:member({sctp, true}, Config) of
-        true  -> accept(sctp);
-        false -> {skip, no_sctp}
-    end.
-
-%% Start multiple accepting transport processes that are connected to
-%% with an equal number of connecting processes using gen_tcp/sctp
-%% directly.
-
--define(PEER_COUNT, 8).
-
 accept(Prot) ->
+    ?TL("accept -> entry with"
+        "~n   Prot: ~p", [Prot]),
+
     Ref = make_ref(),
     true = diameter_reg:add_new({diameter_config, transport, Ref}), %% fake it
     T = {Prot, Ref},
-    [] = ?util:run(?util:scramble(acc(2*?PEER_COUNT, T, []))).
+    Res = ?RUN([{{?MODULE, [init, X, T]}, 15000}
+                || X <- [accept, gen_connect]]),
+
+    ?TL("accept -> done when"
+        "~n   Res: ~p", [Res]),
+    ok.
 
-acc(0, _, Acc) ->
-    Acc;
-acc(N, T, Acc) ->
-    acc(N-1, T, [{?MODULE, [init,
-                            element(1 + N rem 2, {accept, gen_connect}),
-                            T]}
-                 | Acc]).
 
 %% ===========================================================================
-%% tcp_connect/1
-%% sctp_connect/1
+%% connect/1
 %%
 %% Test code accepting, diameter transport connecting.
 
-tcp_connect(_) ->
-    connect(tcp).
-
-sctp_connect(Config) ->
-    case lists:member({sctp, true}, Config) of
-        true  -> connect(sctp);
-        false -> {skip, no_sctp}
-    end.
-
 connect(Prot) ->
+    ?TL("connect -> entry with"
+        "~n   Prot: ~p", [Prot]),
+
     T = {Prot, make_ref()},
-    [] = ?util:run([{?MODULE, [init, X, T]} || X <- [gen_accept, connect]]).
+    Res = ?RUN([{{?MODULE, [init, X, T]}, 15000}
+                 || X <- [gen_accept, connect]]),
+
+    ?TL("connect -> done when"
+        "~n   Res: ~p", [Res]),
+    ok.
+
 
 %% ===========================================================================
 %% reconnect/1
@@ -179,53 +200,111 @@ connect(Prot) ->
 %% doesn't try to establish a new connection until the old one is
 %% broken.
 
-reconnect() ->
-    [{timetrap, {minutes, 4}}].
-
 reconnect({listen, Ref}) ->
+    ?TL("reconnect(listen) -> entry with"
+        "~n   Ref: ~p", [Ref]),
     SvcName = make_ref(),
+    ?TL("reconnect(listen) -> start event logger"),
+    Logger = ?DEL_START("reconnect(listen)", SvcName),
+    ?TL("reconnect(listen) -> start service (~p)", [SvcName]),
     ok = start_service(SvcName),
-    LRef = ?util:listen(SvcName, tcp, [{watchdog_timer, 6000}]),
+    ?TL("reconnect(listen) -> connect"),
+    LRef = ?LISTEN(SvcName, tcp, [{watchdog_timer, 6000}]),
+    ?TL("reconnect(listen) -> wait"),
     [_] = diameter_reg:wait({diameter_tcp, listener, {LRef, '_'}}),
+    ?TL("reconnect(listen) -> register new transport"),
     true = diameter_reg:add_new({?MODULE, Ref, LRef}),
 
     %% Wait for partner to request transport death.
+    ?TL("reconnect(listen) -> abort: await (request for) transport death"),
     TPid = abort(SvcName, LRef, Ref),
 
     %% Kill transport to force the peer to reconnect.
+    ?TL("reconnect(listen) -> kill transport"),
     exit(TPid, kill),
 
     %% Wait for the partner again.
-    abort(SvcName, LRef, Ref);
+    ?TL("reconnect(listen) -> abort: wait for partner again"),
+    Res = abort(SvcName, LRef, Ref),
+
+    ?TL("reconnect(listen) -> stop event logger"),
+    ?DEL_STOP(Logger),
+
+    ?TL("reconnect(listen) -> done when"
+        "~n   Res: ~p", [Res]),
+    ok;
 
 reconnect({connect, Ref}) ->
+    ?TL("reconnect(connect) -> entry with"
+        "~n   Ref: ~p", [Ref]),
+
     SvcName = make_ref(),
+    ?TL("reconnect(connect) -> start event logger"),
+    Logger = ?DEL_START("reconnect(connect)", SvcName),
+    ?TL("reconnect(connect) -> subscribe to service ~p", [SvcName]),
     true = diameter:subscribe(SvcName),
+    ?TL("reconnect(connect) -> start service ~p", [SvcName]),
     ok = start_service(SvcName),
+    ?TL("reconnect(connect) -> wait when"
+        "~n   Svc transports:  ~p"
+        "~n   Svc connections: ~p",
+        [diameter:service_info(SvcName, transport),
+         diameter:service_info(SvcName, connections)]),
     [{{_, _, LRef}, Pid}] = diameter_reg:wait({?MODULE, Ref, '_'}),
-    CRef = ?util:connect(SvcName, tcp, LRef, [{connect_timer, 2000},
-                                              {watchdog_timer, 6000}]),
+    ?TL("reconnect(connect) -> connect when"
+        "~n   Svc transports:  ~p"
+        "~n   Svc connections: ~p",
+        [diameter:service_info(SvcName, transport),
+         diameter:service_info(SvcName, connections)]),
+    CRef = ?CONNECT(SvcName, tcp, LRef, [{connect_timer, 2000},
+                                         {watchdog_timer, 6000}]),
+    ?TL("reconnect(connect) -> connected when"
+        "~n   Svc transports:  ~p"
+        "~n   Svc connections: ~p",
+        [diameter:service_info(SvcName, transport),
+         diameter:service_info(SvcName, connections)]),
 
     %% Tell partner to kill transport after seeing that there are no
     %% reconnection attempts.
+    ?TL("reconnect(connect) -> abort (kill transport)"),
     abort(SvcName, Pid, Ref),
 
     %% Transport goes down and is reestablished.
+    ?TL("reconnect(connect) -> await transport down"),
     ?RECV(#diameter_event{service = SvcName, info = {down, CRef, _, _}}),
+    ?TL("reconnect(connect) -> await transport reconnect"),
     ?RECV(#diameter_event{service = SvcName, info = {reconnect, CRef, _}}),
+    ?TL("reconnect(connect) -> await transport up"),
     ?RECV(#diameter_event{service = SvcName, info = {up, CRef, _, _, _}}),
 
     %% Kill again.
+    ?TL("reconnect(connect) -> abort (kill transport again)"),
     abort(SvcName, Pid, Ref),
 
     %% Wait for partner to die.
+    ?TL("reconnect(connect) -> await partner death"),
     MRef = erlang:monitor(process, Pid),
-    ?RECV({'DOWN', MRef, process, _, _});
+    Res = ?RECV({'DOWN', MRef, process, _, _}),
 
-reconnect(_) ->
+    ?TL("reconnect(connect) -> stop event logger"),
+    ?DEL_STOP(Logger),
+
+    ?TL("reconnect(connect) -> done when"
+        "~n   Res: ~p", [Res]),
+    ok;
+
+reconnect(Prot) ->
+    ?TL("reconnect -> entry with"
+        "~n   Prot: ~p", [Prot]),
     Ref = make_ref(),
-    [] = ?util:run([{?MODULE, [reconnect, {T, Ref}]}
-                    || T <- [listen, connect]]).
+    Res = ?RUN([{{?MODULE, [reconnect, {T, Ref}]}, 240000}
+                || Prot == tcp,  %% ignore sctp
+                   T <- [listen, connect]]),
+
+    ?TL("reconnect -> done when"
+        "~n   Res: ~p", [Res]),
+    ok.
+
 
 start_service(SvcName) ->
     OH = diameter_util:unique_string(),
@@ -263,6 +342,10 @@ abort(SvcName, LRef, Ref)
 %% init/2
 
 init(accept, {Prot, Ref}) ->
+    ?TL("init(accept) -> entry with"
+        "~n   Prot: ~p"
+        "~n   Ref:  ~p", [Prot, Ref]),
+
     %% Start an accepting transport and receive notification of a
     %% connection.
     TPid = start_accept(Prot, Ref),
@@ -276,19 +359,34 @@ init(accept, {Prot, Ref}) ->
     %% Expect the transport process to die as a result of the peer
     %% closing the connection.
     MRef = erlang:monitor(process, TPid),
-    ?RECV({'DOWN', MRef, process, _, _});
+    Res = ?RECV({'DOWN', MRef, process, _, _}),
+
+    ?TL("init(accept) -> done when"
+        "~n   Res: ~p", [Res]),
+    ok;
 
 init(gen_connect, {Prot, Ref}) ->
+    ?TL("init(gen_connect) -> entry with"
+        "~n   Prot: ~p"
+        "~n   Ref:  ~p", [Prot, Ref]),
+
     %% Lookup the peer's listening socket.
-    [PortNr] = ?util:lport(Prot, Ref),
+    [PortNr] = ?LPORT(Prot, Ref),
 
     %% Connect, send a message and receive it back.
     {ok, Sock} = gen_connect(Prot, PortNr),
     Bin = make_msg(),
     ok = gen_send(Prot, Sock, Bin),
-    Bin = gen_recv(Prot, Sock);
+    Bin = gen_recv(Prot, Sock),
+
+    ?TL("init(gen_connect) -> done"),
+    ok;
 
 init(gen_accept, {Prot, Ref}) ->
+    ?TL("init(gen_accept) -> entry with"
+        "~n   Prot: ~p"
+        "~n   Ref:  ~p", [Prot, Ref]),
+
     %% Open a listening socket and publish the port number.
     {ok, LSock} = gen_listen(Prot),
     {ok, PortNr} = inet:port(LSock),
@@ -299,14 +397,22 @@ init(gen_accept, {Prot, Ref}) ->
     {ok, Sock} = gen_accept(Prot, LSock),
     Bin = gen_recv(Prot, Sock),
     ok = gen_send(Prot, Sock, Bin),
-    receive
-        {tcp_closed, Sock} = T ->
-            T;
-        ?SCTP(Sock, {_, #sctp_assoc_change{}}) = T ->
-            T
-    end;
+    _Res = receive
+               {tcp_closed, Sock} = T ->
+                   T;
+               ?SCTP(Sock, {_, #sctp_assoc_change{}}) = T ->
+                   T
+           end,
+
+    ?TL("init(gen_accept) -> done when"
+        "~n   Res: ~p", [_Res]),
+    ok;
 
 init(connect, {Prot, Ref}) ->
+    ?TL("init(connect) -> entry with"
+        "~n   Prot: ~p"
+        "~n   Ref:  ~p", [Prot, Ref]),
+
     %% Lookup the peer's listening socket.
     [{?TEST_LISTENER(_, PortNr), _}]
         = diameter_reg:wait(?TEST_LISTENER(Ref, '_')),
@@ -318,7 +424,10 @@ init(connect, {Prot, Ref}) ->
     %% Send a message and receive it back.
     Bin = make_msg(),
     TPid ! ?TMSG({send, Bin}),
-    Bin = bin(Prot, ?RECV(?TMSG({recv, P}), P)).
+    Bin = bin(Prot, ?RECV(?TMSG({recv, P}), P)),
+
+    ?TL("init(connect) -> done"),
+    ok.
 
 bin(sctp, #diameter_packet{bin = Bin}) ->
     Bin;
@@ -421,18 +530,65 @@ gen_accept(tcp, LSock) ->
 
 gen_send(sctp, Sock, Bin) ->
     {OS, _IS, Id} = getr(assoc),
-    gen_sctp:send(Sock, Id, erlang:unique_integer([positive]) rem OS, Bin);
+    case gen_sctp:send(Sock, Id, erlang:unique_integer([positive]) rem OS, Bin) of
+        ok ->
+            ok;
+        {error, Reason} = ERROR ->
+            Info = try inet:info(Sock) of
+                       I ->
+                           I
+                   catch
+                       C:E:S ->
+                           [{class,  C},
+                            {error,  E},
+                            {stack,  S}]
+                   end,
+            ?TL("Failed (sctp) sending message: "
+                "~n   Reason:        ~p"
+                "~n   Socket:        ~p"
+                "~n   (Socket) Info: ~p", [Reason, Sock, Info]),
+            ERROR
+    end;
 gen_send(tcp, Sock, Bin) ->
-    gen_tcp:send(Sock, Bin).
+    case gen_tcp:send(Sock, Bin) of
+        ok ->
+            ok;
+        {error, Reason} = ERROR ->
+            Info = try inet:info(Sock) of
+                       I ->
+                           I
+                   catch
+                       C:E:S ->
+                           [{class,  C},
+                            {error,  E},
+                            {stack,  S}]
+                   end,
+            ?TL("Failed (tcp) sending message: "
+                "~n   Reason:        ~p"
+                "~n   Socket:        ~p"
+                "~n   (Socket) Info: ~p", [Reason, Sock, Info]),
+            ERROR
+    end.
 
 %% gen_recv/2
 
 gen_recv(sctp, Sock) ->
-    {_OS, _IS, Id} = getr(assoc),
+    {OS, IS, Id} = getr(assoc),
     receive
-        ?SCTP(Sock, {[#sctp_sndrcvinfo{assoc_id = Id}], Bin})
+        ?SCTP(Sock, {[#sctp_sndrcvinfo{assoc_id = I} = INFO], Bin})
           when is_binary(Bin) ->
-            Bin
+            case {I, Id} of
+                {Id, _} -> % assert
+                    Bin;
+                _ ->
+                    ?TL("unexpected assoc id in received info msg:"
+                        "~n   Expected Assoc ID: ~p"
+                        "~n      OS: ~p"
+                        "~n      IS: ~p"
+                        "~n   Received Assoc ID: ~p"
+                        "~n   Info:              ~p", [Id, OS, IS, I, INFO]),
+                    ct:fail({unexpected_assoc_id, I, Id})
+            end
     end;
 gen_recv(tcp, Sock) ->
     tcp_recv(Sock, <<>>).
