@@ -1,7 +1,7 @@
 %% 
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 2004-2020. All Rights Reserved.
+%% Copyright Ericsson AB 2004-2022. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -239,7 +239,7 @@ do_notes({_, Pid}, _Config) ->
 info(suite) ->
     [];
 info(doc) ->
-    ["Testing that we can retreive process info."];
+    ["Testing that we can retrieve process info."];
 info(Config) when is_list(Config) ->
     Pre = fun() ->
                   ?IPRINT("try start note-store"),
